use std::collections::HashMap;

pub type Symbols = HashMap<String, usize>;

pub fn load_symbols(contents: &str) -> anyhow::Result<Symbols> {
    let symbols = contents
        .lines()
        .filter_map(|line| {
            let mut words = line.split_ascii_whitespace();
            match (words.next(), words.next(), words.next()) {
                // Get only two-word lines.
                (Some(address), Some(symbol), None) => {
                    Some((symbol.to_owned(), address.to_owned()))
                }

                _ => None,
            }
        })
        .filter_map(|(symbol, mut address)| {
            if let Some(colon_at) = address.find(':') {
                address.remove(colon_at);
            }

            // Filter out the ones that don't have a hexadecimal number as the first word.
            let snes_address = u32::from_str_radix(&address, 16).ok()?;

            let pc_address = snes_to_pc_address(snes_address);

            Some((symbol, pc_address))
        })
        .collect();

    Ok(symbols)
}

pub fn snes_to_pc_address(snes: u32) -> usize {
    ((snes & 0x7FFF) + ((snes / 2) & 0xFF8000)) as usize
}

pub fn pc_to_snes_address(pc: usize) -> u32 {
    ((pc & 0x7FFF) + ((pc & 0xFF8000) * 2)) as u32
}
